import { Entity, type EntityId } from '@slovo/domain/entity';
import type { Pool } from 'mariadb';

export interface Repository<T> {
  get(id: EntityId): Promise<Entity<T>>;
  save(model: T): Promise<Entity<T>>;
  update(model: Entity<T>): Promise<Entity<T>>;
  delete(model: Entity<T>): Promise<boolean>;
}

export abstract class BaseRepository<T> implements Repository<T> {
  protected pool: Pool;

  public constructor(pool: Pool) {
    this.pool = pool;
  }

  public abstract get(id: EntityId): Promise<Entity<T>>;
  public abstract save(model: T): Promise<Entity<T>>;
  public abstract update(model: Entity<T>): Promise<Entity<T>>;
  public abstract delete(model: Entity<T>): Promise<boolean>;
}
