import { BaseRepository } from '@slovo/domain/repository';
import { Entity, type EntityId } from '@slovo/domain/entity';
import type { Image } from '@slovo/models/image';
import type { PoolConnection } from 'mariadb';

export class ImageRepository extends BaseRepository<Image> {
  public async get(id: EntityId): Promise<Entity<Image>> {
    const conn: PoolConnection = await this.pool.getConnection(); 

    const query = `SELECT * FROM images WHERE id = ?`;
    const rows = await conn.query(query, [ id ]);

    if (!rows.length) {
      throw Error("Unable to find Image.");
    }

    const row = rows.shift();

    await conn.release();
    return new Entity(row.id, {
      url: new URL(row.url),
      title: row.title,
      width: row.width,
      height: row.height 
    } as Image);
  }

  public async save(image: Image): Promise<Entity<Image>> {
    const conn: PoolConnection = await this.pool.getConnection();

    const query = `
      INSERT INTO images (url, title, width, height) 
      VALUES (?, ?, ?, ?)
    `;
    const rows = await conn.query(query, [
      image.url.toString(),
      image.title,
      image.width,
      image.height
    ]);

    if (!rows.insertId) {
      throw Error("Unable to create Image");
    }

    await conn.release();
    return new Entity(rows.insertId, image);
  }

  public async update(image: Entity<Image>): Promise<Entity<Image>> {

  }

  public async delete(image: Entity<Image>): Promise<boolean> {

  }
}
