import { BaseRepository } from '@slovo/domain/repository';
import { Entity, type EntityId } from '@slovo/domain/entity';
import { type User } from '@slovo/models/user';
import { type PoolConnection } from 'mariadb';

export class UserRepository extends BaseRepository<User> {
  public async get(id: EntityId): Promise<Entity<User>> {
    const conn: PoolConnection = await this.pool.getConnection();

    const query = `SELECT * FROM users WHERE id = ?`;
    const rows = await conn.query(query, [ id ]);

    if (!rows.length) {
      throw Error("Unable to find User");
    }

    const row = rows.shift();

    const user: User = {
      name: row.name,
      email: row.email,
      password: row.password
    };

    await conn.release();
    return new Entity(id, user);
  }

  public async getAll(): Promise<Set<Entity<User>>> {
    const conn: PoolConnection = await this.pool.getConnection();

    const query = `SELECT * FROM users`;
    const rows = await conn.query(query);

    const users = new Set<Entity<User>>();

    if (!rows.length) {
      return users;
    }

    for (const row of rows) {
      const user: Entity<User> = new Entity(row.id, {
        email: row.email,
        name: row.name,
      });
      users.add(user);
    }

    return users;
  }

  public async save(user: User): Promise<Entity<User>> {
    const conn: PoolConnection = await this.pool.getConnection();

    const query = `INSERT INTO users (name, email, password) VALUES (?, ?, ?)`;
    const rows = await conn.query(query, [ user.name, user.email, user.password ]);

    if (!rows.length) {
      throw Error("Unable to create User");
    }

    await conn.release();
    return new Entity(rows.insertId, user);
  }

  public async update(user: Entity<User>): Promise<Entity<User>> {
    const conn: PoolConnection = await this.pool.getConnection();
    
    const query = `UPDATE user SET name = ?, email = ?, password = ? WHERE id = ?`;
    const results = await conn.query(query, [
      user.model.name,
      user.model.email,
      user.model.password,
      user.getId()
    ]);

    if (!results.length) {
      throw Error("Unable to update the User");
    }

    await conn.release();
    return user;
  }

  /// Don't delete users
  public async delete(user: Entity<User>): Promise<boolean> {
    return true;
  }
}
